#ifndef ANCHOR_MOVER
#define ANCHOR_MOVER

#include <core/types.hh>
#include <core/pose/Pose.hh>
#include <core/kinematics/MoveMap.hh>
#include <core/fragment/FragSet.hh>
#include <core/fragment/FrameList.hh>

#include <utility/vector1.fwd.hh>

#include <protocols/moves/Mover.hh>
#include <protocols/abinitio/FragmentMover.hh>
#include <protocols/abinitio/SmoothFragmentMover.hh>


typedef utility::vector1<core::Real> ScoreList;


class AnchorMover : virtual public protocols::abinitio::ClassicFragmentMover {
  public:
    
    AnchorMover();
    
    AnchorMover(core::fragment::FragSetCOP fragset,
                protocols::abinitio::FragmentCostOP cost );

    AnchorMover(core::fragment::FragSetCOP fragset,
                core::kinematics::MoveMapCOP movemap,
                protocols::abinitio::FragmentCostOP cost);
    
    ~AnchorMover();

    bool apply_fragment();

  protected:
    
    AnchorMover(core::fragment::FragSetCOP fragset, 
                core::kinematics::MoveMapCOP movemap,
                protocols::abinitio::FragmentCostOP cost, 
                std::string move_type);

    // virtual bool choose_fragment(core::fragment::FrameList const&, 
    //                               core::pose::Pose const&, 
    //                               Size& frame_num, 
    //                               Size& frag_num) const;
    
    virtual bool apply( core::pose::Pose&, 
                        Size pos) const; //apply at particular position
    
    
    virtual std::string get_name() const;
    
   //  virtual bool apply_frames(core::pose::Pose& pose, 
   //                            core::fragment::FrameList const& frames ) const;
  
  private:
    protocols::abinitio::FragmentCostOP cost_;

    //choose randomly fragments that are below cutoff_
    core::Real cutoff_;

    bool repack_;

};

#endif
