#ifndef RELAX_WRAPPER 
#define RELAX_WRAPPER 

#include <protocols/relax/RelaxProtocolBase.hh>
#include <protocols/moves/Mover.hh>

#include <core/kinematics/MoveMap.fwd.hh>
#include <core/pose/Pose.fwd.hh>

#include <core/scoring/ScoreFunction.fwd.hh>
#include <protocols/moves/MoverCreator.hh>
#include <protocols/moves/MoverCreator.hh>

#include <protocols/checkpoint/CheckPointer.hh>




#include <premr_options.hh>
#include <mr.hh>

#include <utility/string_util.hh>
#include <string>


// procedure:
// 1. Packing side chains
// 2. Do minimize
// 2. Do molecular replacement

struct RelaxScriptCommand {
  RelaxScriptCommand() {
    command="";
    param1=0;
    param2=0;
    param3=0;
    param4=0;
    nparams=0;
  }
  std::string command;
  core::Real  param1;
  core::Real  param2;
  core::Real  param3;
  core::Real  param4;
  core::Size  nparams;
};

class AnchorRelax: public protocols::relax::RelaxProtocolBase {

  private:
    typedef core::pose::Pose Pose; 
    typedef core::scoring::ScoreFunctionOP FullScorefxnOP; 

  private:
    // core::scoring::ScoreFunctionOP scorefxn_;
    MolecularReplacement mr_; 

    std::string min_type_;
    core::Size default_repeats_;
    bool repack_;

    /// @brief Apply Ramady(tm) fix to residues with bad rama scores ?
    bool ramady_;

    std::vector<RelaxScriptCommand> templates_;
    
    // utility::Tag::TagPtr movemap_tag_;
    // protocols::checkpoint::CheckPointer checkpoints_;

  public:
    
    AnchorRelax( core::scoring::ScoreFunctionOP scorefxn, core::Size standard_repeats );
    
    virtual ~AnchorRelax();

    void set_molecular_replacement(MolecularReplacement mr);
    
    MolecularReplacement get_molecular_replacement();

    virtual void apply(Pose &pose); 

    virtual std::string get_name() const;

    void make_relax_template( core::Size standard_repeats);

    void do_minimize_and_mr(Pose &pose, core::Real tolerance, 
                            core::kinematics::MoveMapOP local_movemap, 
                            FullScorefxnOP local_scorefxn);
    
    void set_and_create_mr_defaults();

    void relax_directly_from_rosetta(const Pose pose, FullScorefxnOP full_scorefxn); 

};

#endif
