#ifndef SIMULATED_ANNEALING 
#define SIMULATED_ANNEALING

#include <exception>
#include <string>
#include <map>
#include <vector>

#include <protocols/init.hh> // Rosetta initialization
#include <protocols/abinitio/AbrelaxApplication.hh>
#include <protocols/abinitio/ClassicAbinitio.hh>
#include <core/types.hh>

#include <core/options/option.hh>
#include <core/options/keys/loops.OptionKeys.gen.hh>

#include <core/pose/Pose.hh>
#include <core/io/pdb/pose_io.hh>
#include <core/fragment/FragmentIO.hh>
#include <core/fragment/FragSet.hh>
#include <protocols/abinitio/FragmentMover.hh>
// #include <numeric/random/random.hh>


#include <core/scoring/ScoreFunction.fwd.hh>
#include <core/scoring/ScoreType.hh>
#include <core/scoring/ScoreFunctionFactory.hh>

#include <protocols/abinitio/KinematicAbinitio.hh>
#include <protocols/relax/ClassicRelax.hh>
#include <protocols/relax/FastRelax.hh>
#include <protocols/relax/util.hh>

#include <evaluation.hh>
#include <r_mover.hh>
#include <loop_wrapper.hh>

#include <disorder_residue_position.hh>
#include<init_rosetta.hh>
#include<mr.hh>
#include<structure_util.hh>


using namespace std;
using namespace core::options;
using namespace core::options::OptionKeys;

class DisorderModel {

  private: //typedef
    typedef core::pose::Pose Pose;
 
  private:  //variables

    core::pose::Pose start_pose_;
    core::scoring::ScoreFunctionOP curr_score_fxn_;

    RMover r_mover_;
    Model_Evaluation model_evaluation_;
    DisorderResidues disorder_residues_;


    // new setups
    string disorder_score_path;
    string input_model_path;
    string path2native_;
    Rosetta rosetta;
    MolecularReplacement mr_;
    Pose crystal_pose_;
    unsigned int num_of_models_;

    float max_disorder_score_;
    float min_disorder_score_;

  public:

    DisorderModel();

    DisorderModel(string new_disorder_score_path, string new_input_model_path);

    void set_num_of_models(string new_num_of_models);

    unsigned int get_num_of_models(); 

    float calculate_temperature(float cur_rsd_disorder_score);

    bool initialize(Rosetta new_rosetta);

    string check_ss( string const sec_ss, int pos); 

    void remodel_disorder_rsds();

    void remodel_disorder_rsds(int job_id);

    void remodel_disorder_rsds(int job_id, float &phaser_score, char* &pose_id);

    void remodel_less_disorder_rsds();

    string get_file_name_wo_ext();

    void set_native_pose(const string path2native); 

    void set_MR(const MolecularReplacement mr);
};

# endif
