#include<disorder_residue_position.hh>
#include<util.hh>

#include<string>
#include<iostream>
#include<map>

using namespace std;
static numeric::random::RandomGenerator ruRG(20200);

bool DisorderResidues::read_disorder_scores(string score_file, map<int, float> &disorder_scores ) {
  string line;
  ifstream infile(score_file.c_str(), ifstream::in);
  if(infile.is_open()) {
    while(getline(infile, line)) {
      stringstream ss(line);
      string entry;
      int index   = -1;  //never possible
      float value = 0.0; //ideal 
      ss >> index >> value;
      disorder_scores.insert(pair<int, float> (index, value));
    }
    infile.close();
    return true;
  }
  else {
    cout << "Unable to open disorder score file." << endl;
    return false;
  }
}
  
void DisorderResidues::generate_disorder_rsds(map<int, float> disorder_scores) {
  float sum = 0.0, avg = 0.0;
  map<int, float> rsds_to_be_built;
  map<int, float>::iterator it;
    
  for(it = disorder_scores.begin(); it != disorder_scores.end(); it++)
    sum += (*it).second;
  avg = sum / float(disorder_scores.size());
  int num_rsd_to_be_build = 0;
  for(it = disorder_scores.begin(); it != disorder_scores.end(); it++) {
    if (float((*it).second) >= avg) {
      rsds_to_be_built.insert(pair<int, float> ((*it).first, (*it).second));
      num_rsd_to_be_build++;
    }
  }
  discretize(rsds_to_be_built); 
}

void DisorderResidues::generate_each_disorder_rsds_score( map<int, float> disorder_scores) {
  map<int, float>::iterator it;
  int count = 0; 
  for(it = disorder_scores.begin(); it != disorder_scores.end(); it++) 
    all_disorder_rsds_.push_back(DisorderResidue(++count, (*it).first, (*it).first,  (*it).second));
}

list<DisorderResidue> DisorderResidues::get_disorder_rsds(int start_rsd_pos, int ending_rsd_pos) {
  list<DisorderResidue> selected_disorder_rsds_;
  list<DisorderResidue>::iterator it;
  bool is_in = false;
  for(it = all_disorder_rsds_.begin(); it != all_disorder_rsds_.end(); it++) {
    if((*it).get_start_rsd_pos() == start_rsd_pos) is_in = true;
    if((*it).get_start_rsd_pos() > ending_rsd_pos) { 
      is_in = false;
      return selected_disorder_rsds_;
    }
    if(is_in) selected_disorder_rsds_.push_back(*(it));
  }
  return selected_disorder_rsds_;
} 

list<DisorderResidue> DisorderResidues::get_less_disorder_rsds( ) {
  list<DisorderResidue> less_disorder_rsds_;
  list<DisorderResidue>::iterator it;
  float sum = 0.0, avg = 0.0;

  for(it = all_disorder_rsds_.begin(); it != all_disorder_rsds_.end(); it++) {
    sum += (*it).get_disorder_score(); 
  }
  avg = sum /all_disorder_rsds_.size();

  for(it = all_disorder_rsds_.begin(); it != all_disorder_rsds_.end(); it++) {
    if((*it).get_disorder_score() < avg)
      less_disorder_rsds_.push_back(*it);
  }
  return less_disorder_rsds_;
} 

list<DisorderResidue> DisorderResidues::get_highly_disorder_rsds( ) {
  list<DisorderResidue> less_disorder_rsds_;
  list<DisorderResidue>::iterator it;
  float sum = 0.0, avg = 0.0;

  for(it = all_disorder_rsds_.begin(); it != all_disorder_rsds_.end(); it++) {
    sum += (*it).get_disorder_score(); 
  }
  avg = sum /all_disorder_rsds_.size();

  for(it = all_disorder_rsds_.begin(); it != all_disorder_rsds_.end(); it++) {
    if((*it).get_disorder_score() >= avg) {
      less_disorder_rsds_.push_back(*it);
    }
  }
  return less_disorder_rsds_;
}

void DisorderResidues::discretize (map<int, float> rsds_to_be_built ) {
  map<int, float>::iterator it; bool isfirst = true; float disoder_score = 0.0;
  int first_res_pos = 0, last_res_pos = 0, count = 0;
  for(it = rsds_to_be_built.begin(); it != rsds_to_be_built.end(); it++) {
    if (isfirst) { 
      last_res_pos  = it->first;
      first_res_pos = it->first;
      disoder_score += it->second;
      isfirst       = false;
    }
    if ((it->first - last_res_pos) > 1) {
      disorder_rsds_.push_back(DisorderResidue(++count, first_res_pos, last_res_pos, 
                                  disoder_score /(last_res_pos-first_res_pos + 1)));
      first_res_pos = it->first;
      last_res_pos  = it->first;
      disoder_score = 0.0;
    }
    else
      last_res_pos  = it->first;
      disoder_score += it->second;
  }
  disorder_rsds_.push_back(DisorderResidue(++count, first_res_pos, last_res_pos, 
                                   disoder_score / (last_res_pos-first_res_pos + 1)));
}

DisorderResidue DisorderResidues::choose_rsds_to_remodel(list<DisorderResidue> dis_rsds) {
  list<DisorderResidue>::iterator it_dis_rsds;
  DisorderResidue disorder_residue;
  float total_disorder_score = 0;
  for( it_dis_rsds = dis_rsds.begin(); it_dis_rsds != dis_rsds.end(); it_dis_rsds++ ) {
    total_disorder_score += (*it_dis_rsds).get_disorder_score();
  }
  float rg = ruRG.uniform();
  float fortune = rg * total_disorder_score;
  for( it_dis_rsds = dis_rsds.begin(); it_dis_rsds != dis_rsds.end(); it_dis_rsds++ ) {
    fortune -= (*it_dis_rsds).get_disorder_score();
    disorder_residue = (*it_dis_rsds);
    if (fortune <= 0.0)  {
      return disorder_residue;
    }
  }
  return disorder_residue; 
}

unsigned int DisorderResidues::choose_local_rsds_to_remodel(list<DisorderResidue> dis_rsds, float total_disorder_score) {
  list<DisorderResidue>::iterator it_dis_rsds;
  unsigned int selected_rsd_pos = 1;
  float rg = ruRG.uniform();
  float fortune = rg * total_disorder_score;
  for( it_dis_rsds = dis_rsds.begin(); it_dis_rsds != dis_rsds.end(); it_dis_rsds++ ) {
    fortune -= (*it_dis_rsds).get_disorder_score();
    selected_rsd_pos = (*it_dis_rsds).get_start_rsd_pos();
    if (fortune <= 0.0)  {
      return selected_rsd_pos;
    }
  }
  return selected_rsd_pos; 
}

bool DisorderResidues::compare(const pair<int, float> score1, const pair<int, float> score2) {
  return score1.second < score2.second;
}

float DisorderResidues::get_max_disorder_score() {
  return max_disorder_score_;
}

float DisorderResidues::get_min_disorder_score() {
  return min_disorder_score_;
}

void DisorderResidues::set_max_min_disorder_score( map<int, float> disorder_scores) {
  map<int, float>::const_iterator it_max = StatisticalUtil::map_max_element(disorder_scores); 
  map<int, float>::const_iterator it_min = StatisticalUtil::map_min_element(disorder_scores); 
  min_disorder_score_ = (*it_min).second; 
  max_disorder_score_ = (*it_max).second; 
  cout<<"Maximum disorder score "<<max_disorder_score_
      <<" and minimum disorder score "<<min_disorder_score_<<endl;
}

float DisorderResidues::get_disorder_score(int rsd_pos) {
  list<DisorderResidue>::iterator it_dis_rsds;
  for(it_dis_rsds = all_disorder_rsds_.begin(); it_dis_rsds != all_disorder_rsds_.end(); it_dis_rsds++ ) {
    if(((*it_dis_rsds).get_start_rsd_pos() == rsd_pos) && ((*it_dis_rsds).get_end_rsd_pos() == rsd_pos)) {
      return (*it_dis_rsds).get_disorder_score();
    }
  }
  return -1.0; // never possible
}

float DisorderResidues::get_total_disorder_score(list<DisorderResidue> dis_rsds) {
  float total_disorder_score = 0.0;
  list<DisorderResidue>::iterator it_dis_rsds;
  for( it_dis_rsds = dis_rsds.begin(); it_dis_rsds != dis_rsds.end(); it_dis_rsds++ ) {
    total_disorder_score += (*it_dis_rsds).get_disorder_score();
  }
  return total_disorder_score;
}

void DisorderResidues::show() {
  list<DisorderResidue>::iterator iter_disorder_rsd;
  for ( iter_disorder_rsd= disorder_rsds_.begin(); iter_disorder_rsd != disorder_rsds_.end(); iter_disorder_rsd++) {
    iter_disorder_rsd->show();
  }
}
    
 
