#ifndef DISORDER_RSDS_HH 
#define DISORDER_RSDS_HH 


#include <numeric/random/random.hh>

# include <iostream>
# include <fstream>
# include <sstream>
# include <map>
# include <list>
# include <string>
# include <algorithm>

using namespace std;

class DisorderResidue {
  
  private:
    int   index_;
    int   start_rsd_pos_;
    int   end_rsd_pos_;
    float disorder_score_;

  public:
    DisorderResidue() {
      index_          = 0; 
      start_rsd_pos_  = 0; 
      end_rsd_pos_    = 0; 
      disorder_score_ = 0.0; 
    }

    DisorderResidue(int index, int start_rsd_pos, int end_rsd_pos, float disorder_score) {
      index_          = index; 
      start_rsd_pos_  = start_rsd_pos;
      end_rsd_pos_    = end_rsd_pos;
      disorder_score_ = disorder_score;
    }

    void show() {
      // cout << "Disorder residues: " <<endl
      //      << "Index " << " start residue position " <<" ending residue position " <<" disorder score " <<endl
      cout<< index_ <<" " << start_rsd_pos_ << " " <<end_rsd_pos_ << " " <<disorder_score_ <<endl;
    }
    
    bool is_in_between(int rsd_pos) {
      if( rsd_pos >= start_rsd_pos_ && rsd_pos <= end_rsd_pos_)
        return true;
      else
        return false;
    }
    
    int get_start_rsd_pos() {
      return start_rsd_pos_;
    }

    int get_end_rsd_pos() {
      return end_rsd_pos_;
    }

    float get_disorder_score() {
      return disorder_score_;
    }
};


class DisorderResidues { //ROS::better classname

  private:
    list<DisorderResidue> disorder_rsds_;
    list<DisorderResidue> all_disorder_rsds_;
    float min_disorder_score_, max_disorder_score_;

  public:
    list<DisorderResidue> get_disorder_rsds() {
      return disorder_rsds_;
    }

    list<DisorderResidue> get_all_disorder_rsds() {
      return all_disorder_rsds_;
    }

  private:
    void discretize (map<int, float> rsds_to_be_built );

  public:
    static bool read_disorder_scores( string score_file, map<int, float> &disorder_scores );

    void generate_disorder_rsds(map<int, float> disorder_scores ); 
    
    void generate_each_disorder_rsds_score( map<int, float> disorder_scores ); 

    list<DisorderResidue> get_disorder_rsds(int start_rsd_pos, int ending_rsd_pos);
    
    list<DisorderResidue> get_less_disorder_rsds( );
    
    list<DisorderResidue> get_highly_disorder_rsds( );
    
    DisorderResidue choose_rsds_to_remodel(list<DisorderResidue> dis_rsds); 

    unsigned int choose_local_rsds_to_remodel(list<DisorderResidue> dis_rsds, float total_disorder_score);

    float get_total_disorder_score(list<DisorderResidue> dis_rsds); 

    float get_disorder_score(int rsd_pos);

    bool compare(const pair<int, float> score1, const pair<int, float> score2);

    float get_max_disorder_score(); 

    float get_min_disorder_score(); 

    void set_max_min_disorder_score( map<int, float> disorder_scores ); 

    void show();
    
};


#endif
