#ifndef EVALUATION 
#define EVALUATION 

#include <core/pose/Pose.hh>
#include <core/scoring/ScoreType.hh>
#include <core/scoring/ScoreFunction.fwd.hh>
#include <core/scoring/ScoreFunctionFactory.hh>
// #include <core/util/Tracer.hh>
#include <protocols/moves/MonteCarlo.hh>
#include <core/options/keys/abinitio.OptionKeys.gen.hh>

// using namespace OptionKeys;

// static core::util::Tracer tr("preMR.evaluation");

class Model_Evaluation {

  private:    
    core::scoring::ScoreFunctionOP curr_score_fxn;  //score0
    core::scoring::ScoreFunctionOP score_stage1_;   //score0
    core::scoring::ScoreFunctionOP score_stage2_;   //score1
    core::scoring::ScoreFunctionOP score_stage3a_;  //score2
    core::scoring::ScoreFunctionOP score_stage3b_;  //score5
    core::scoring::ScoreFunctionOP score_stage4_;   //score3
    core::scoring::ScoreFunctionOP score_stage5_;   //score3 //vats lets try score3 first
    
    protocols::moves::MonteCarloOP mc_;
    //temperature
    core::Real temperature_;

    enum StageID {
      ALL_STAGES = 0,
      STAGE_1,
      STAGE_2,
      STAGE_3a,
      STAGE_3b,
      STAGE_4,
      STAGE_5
    };

  public:

    Model_Evaluation() {
    }

    void set_defaults(core::pose::Pose const& pose) {
      temperature_ = 1.5;
      set_default_scores();
      curr_score_fxn = score_stage4_;
      set_default_mc(pose, *curr_score_fxn);
    }

    bool mc_evaluate_model(core::pose::Pose & pose) {
      std::string const type = "trial"; // temporary solution
      core::Real a = 1.0;               // temporary solution
      return mc_->boltzmann(pose, type, a);
    }

    core::scoring::ScoreFunction get_curr_scorefxn() {
      return *curr_score_fxn; 
    }

    core::scoring::ScoreFunctionOP get_curr_scorefxn_op() {
      return curr_score_fxn; 
    }
  
  private:
    
    void set_default_mc( core::pose::Pose const&pose, core::scoring::ScoreFunction const &scorefxn ) {
      protocols::moves::MonteCarloOP mc = new protocols::moves::MonteCarlo( pose, scorefxn, temperature_);
      mc_ = mc;
    }

    void set_default_scores() {
      using namespace core::scoring;
      using namespace core::options;
      using namespace core::options::OptionKeys;
      // tr.Debug << "creating standard scoring functions" << std::endl;
      if ( option[ OptionKeys::abinitio::stage1_patch ].user() ) {
        score_stage1_  = ScoreFunctionFactory::create_score_function("score0", option[ OptionKeys::abinitio::stage1_patch ]() );
      } else {
        score_stage1_  = ScoreFunctionFactory::create_score_function("score0" );
      }
      if ( option[ OptionKeys::abinitio::stage2_patch ].user() ) {
        score_stage2_  = ScoreFunctionFactory::create_score_function("score1", option[ OptionKeys::abinitio::stage2_patch ]() );
      } else {
        score_stage2_  = ScoreFunctionFactory::create_score_function("score1" );
      }
      if ( option[ OptionKeys::abinitio::stage3a_patch ].user() ) {
        score_stage3a_ = ScoreFunctionFactory::create_score_function("score2", option[ OptionKeys::abinitio::stage3a_patch ]() );
      } else {
        score_stage3a_ = ScoreFunctionFactory::create_score_function( "score2" );
      }
      if ( option[ OptionKeys::abinitio::stage3b_patch ].user() ) {
        score_stage3b_ = ScoreFunctionFactory::create_score_function( "score5", option[OptionKeys::abinitio::stage3b_patch ]() );
      } else {
        score_stage3b_ = ScoreFunctionFactory::create_score_function( "score5" );
      }
      if ( option[ OptionKeys::abinitio::stage4_patch ].user() )
          {
        score_stage4_  = ScoreFunctionFactory::create_score_function( "score3", option[OptionKeys::abinitio::stage4_patch ]() );
      } else {
        score_stage4_  = ScoreFunctionFactory::create_score_function( "score3" );
      }
      if ( option[ OptionKeys::abinitio::stage5_patch].user() ) { //vats
        score_stage5_  = ScoreFunctionFactory::create_score_function( "score3", option[OptionKeys::abinitio::stage5_patch ]() );
      } else {
        score_stage5_  = ScoreFunctionFactory::create_score_function( "score3" );
      }
      if ( option[OptionKeys::abinitio::override_vdw_all_stages ] ) {
        set_score_weight( core::scoring::vdw, option[OptionKeys::abinitio::vdw_weight_stage1 ], ALL_STAGES );
      }
    }

    /// @brief sets a score weight for all stages of abinitio
    void set_score_weight( core::scoring::ScoreType type, core::Real setting, StageID stage ) {
      //tr.Debug << "set score weights for ";
      if ( stage == ALL_STAGES ) std::cout<<"all stages "; // tr.Debug << "all stages ";
      else std::cout<< "stage " << (stage <= STAGE_3a ? stage : (stage-1 ) ) << ( stage == STAGE_3b ? "b " : " " );
      std::cout << core::scoring::name_from_score_type(type) << " " << setting << std::endl;
      // else tr.Debug << "stage " << (stage <= STAGE_3a ? stage : (stage-1 ) ) << ( stage == STAGE_3b ? "b " : " " );
      // tr.Debug << scoring::name_from_score_type(type) << " " << setting << std::endl;
      if (score_stage1_  && ( stage == STAGE_1  || stage == ALL_STAGES)) score_stage1_ ->set_weight(type, setting);
      if (score_stage2_  && ( stage == STAGE_2  || stage == ALL_STAGES)) score_stage2_ ->set_weight(type, setting);
      if (score_stage3a_ && ( stage == STAGE_3a || stage == ALL_STAGES)) score_stage3a_->set_weight(type, setting);
      if (score_stage3b_ && ( stage == STAGE_3b || stage == ALL_STAGES)) score_stage3b_->set_weight(type, setting);
      if (score_stage4_  && ( stage == STAGE_4  || stage == ALL_STAGES)) score_stage4_ ->set_weight(type, setting);
      if (score_stage5_  && ( stage == STAGE_5  || stage == ALL_STAGES)) score_stage5_ ->set_weight(type, setting);//vats
    }
};

#endif
