#ifndef EVOLUTIONARY_HH 
#define EVOLUTIONARY_HH 

#include <string>

//Rosetta Libraries
#include <protocols/init.hh> // Rosetta initialization
#include <core/fragment/FragmentIO.hh>
#include <core/fragment/FragSet.hh>
#include <core/conformation/Conformation.hh>
#include <core/options/option.hh>
#include <core/options/keys/in.OptionKeys.gen.hh>

#include <eo>
//#include <algo/moTS.h>
#include <oldmo>
#include <peo>
#include <string>
//#include<mo>
//#include "Move.h"
#include "PstructFunctions.h"
//#include "pstructPackUnpack.h"
#include "proteinPerturbation.h"
#include "Pstruct.h"
#include "TsFragmentMover.h"
#include "globalHCExplorer.h"
#include "olympusHCExplorer.h"
#include "TsMoveLoopExplorer.h"
#include "proteinSmallMover.h"
#include "proteinWindowTabuList.h"
#include "proteinAspirationCriterion.h"
#include "proteinTS.h"
#include "proteinILS.h"
#include "angularStats.h"
#include "conformationalSample.h"
#include "moveOperators.h"
#include "singleResidueHCExplorer.h"
#include "singleFragmentResidueHCExplorer.h"
//#include "IndexedInsertionNeighbor.h"
#include <algo/moILS.h>

#include "wrong_regions.hh"

using namespace std;

class evolution {
  
  private:
    core::fragment::FragSetOP fragDB9;
    core::fragment::FragSetOP fragDB3;
    string model_path;
    string dbpath;
    RebuildFragment badregion;

  public:
    evolution() {
    }

    evolution(core::fragment::FragSetOP fragDB9_, 
                            core::fragment::FragSetOP fragDB3_, string model_path_, 
                            RebuildFragment badregion_, string dbpath_ ) {
      fragDB9    = fragDB9_;
      fragDB3    = fragDB3_;
      model_path = model_path_;
      badregion  = badregion_;
      dbpath     = dbpath_;
    }

    void run( int argc, char* argv [] ) {
      protocols::init(argc, argv);
      // rng.reseed(time(0) + getpid());
      TsFragmentMover mover(fragDB9, 1, "tsFragmentMover");
    
      // Initialization
      Pstruct pstruct;
      pstruct.create_from_pdb(model_path.c_str(), dbpath.c_str());
    
      PstructEval pstruct_eval;
      proteinMoveInit <TsFragmentMover> move_init;
      proteinRandMoveNext <TsFragmentMover> move_next(mover);
      proteinRandWindowNext <TsFragmentMover> window_next(mover,mover.getWindowNb());
      proteinMoveBestImprSelect <MoveKey, Pstruct> move_select;
     
      moFitComparator <Pstruct> comparator;
      moGenSolContinue<Pstruct> continuBTS(1);   //(param.bigtabuMaxIter);
      moGenSolContinue<Pstruct> continu3frag(1); // param.frag3MaxIter);
      moGenSolContinue<Pstruct> continuILS(10);  // param.ILSmaxIter);
      proteinPerturbation<TsFragmentMover, Pstruct, MoveKey> perturbation(mover, 2); //param.perturbDegree);
      proteinSmallMover bbMover;
      moSteadyFitSolContinue <Pstruct> continuCS(2000, 100);
      conformationalSample<Pstruct, proteinSmallMover> confSampling(bbMover, continuCS, pstruct_eval); 
      globalHCExplorer<TsFragmentMover, Pstruct, MoveKey> explorer2(mover, move_init, window_next, move_next);
    
      proteinTS <TsFragmentMover, Pstruct> bigtabuSearch(explorer2, continuBTS, pstruct_eval);
      proteinTS <TsFragmentMover, Pstruct> frag3Search(explorer2, continu3frag, pstruct_eval);
      proteinILS<TsFragmentMover, Pstruct, MoveKey> iterated_local_search (bigtabuSearch, perturbation, pstruct_eval, continuILS);
      proteinILS<TsFragmentMover, Pstruct, MoveKey> iterated_local_search3Frag (frag3Search, perturbation, pstruct_eval, continuILS) ;
    
      // conformational sampling
      mover.initPose(pstruct);
      pstruct.change_score("/home/rojan/software/rosetta/rosetta32/rosetta-3.2/rosetta_database/scoring/weights/score2.wts");
      pstruct_eval(pstruct);
      iterated_local_search(pstruct);
      confSampling(pstruct);
      pstruct.change_score("/home/rojan/software/rosetta/rosetta32/rosetta-3.2/rosetta_database/scoring/weights/score3.wts");
      pstruct.invalidate();
      iterated_local_search(pstruct);
      confSampling(pstruct);
      pstruct.test();
      pstruct.invalidate();
      iterated_local_search(pstruct);
      confSampling(pstruct);
      pstruct.test();
      // pstruct.write_angleKeys(param.outputKeys.c_str());
      mover.set_nbFrag(100);
      mover.set_fragments(fragDB3);
      pstruct.invalidate();
      move_next.setRSize(100);
      iterated_local_search(pstruct);
      pstruct_eval(pstruct);
      confSampling(pstruct);
      pstruct.test();
      pstruct.myPose.dump_pdb("a.pdb"); // param.outputPdb.c_str());
      pstruct.write_fitness("fitness.log"); //param.outputFitness.c_str());
      pstruct.write_contrib("fitness.contrib"); //param.fitnessContrib.c_str());
    }

};

#endif
