#ifndef LOOP_WRAPPER
#define LOOP_WRAPPER

#include <core/pose/Pose.hh>
#include <protocols/loops/Loop.hh>
#include <protocols/loops/Loops.hh>
#include <protocols/loops/looprelax_protocols.hh>

#include <utility/vector1_bool.hh>
#include <utility/vector1.fwd.hh>

#include <core/scoring/ScoreFunction.hh>
#include <core/types.hh>

// package headers
#include <protocols/loops/IndependentLoopMover.hh>
#include <protocols/moves/Mover.hh>
#include <protocols/loops/LoopMover.hh>


// project headers
#include <core/types.hh>
#include <core/fragment/FragSet.fwd.hh>
#include <core/kinematics/MoveMap.fwd.hh>
#include <core/pose/Pose.fwd.hh>
#include <core/scoring/ScoreFunction.fwd.hh>

#include <disorder_residue_position.hh>

class RosettaLoopWrapper {

  private: //attributes
    typedef core::Size Size;
  
  private:
    protocols::loops::Loops loops_;
    core::scoring::ScoreFunctionOP cur_scorefxn_;
    
  public: //methods
    
    RosettaLoopWrapper(); 
    
    RosettaLoopWrapper(core::scoring::ScoreFunctionOP scorefxn); 

    ~RosettaLoopWrapper();

    void setup_loop(Size start_pos, Size end_pos);
    
    void apply_rebuild_loop(core::pose::Pose & pose, Size start_res_pos, Size end_res_pos);
    
};
   
class CCD_AnchorMover: public protocols::loops::IndependentLoopMover {  
  
  public:

    // CCD_AnchorMover();

    CCD_AnchorMover( protocols::loops::Loops loops_in );

    CCD_AnchorMover( protocols::loops::Loops loops_in, core::scoring::ScoreFunctionOP scorefxn);

    CCD_AnchorMover( protocols::loops::Loops loops_in, core::scoring::ScoreFunctionOP scorefxn, core::fragment::FragSetOP fragset);

    ~CCD_AnchorMover();

    virtual std::string get_name() const;

    virtual protocols::moves::MoverOP clone() const;

    inline bool randomize_loop() const {
      return randomize_loop_;
    }

    /// @brief indicate whether loops should be randomized prior to modeling
    inline void randomize_loop( bool const flag ) {
      randomize_loop_ = flag;
    }
    
    void set_default_settings() {
      randomize_loop_ = true;
    }

    string int2string(int number); 
    
    //ROS:: derived class of Independent loop mover to avoid abstract defintion of class 
    protocols::loops::LoopResult model_loop(core::pose::Pose &pose, protocols::loops::Loop const & loop);

    bool remodel_using_CCD(core::pose::Pose &pose, protocols::loops::Loop const & loop, DisorderResidues disorder_residues);

    void build_loop(core::pose::Pose &pose, protocols::loops::Loop const &loop, DisorderResidues disorder_rsds);

  private:
  
    /// @brief randomize loops prior to performing loop modeling?
    bool randomize_loop_;
   

};


// void fast_ccd_close_loops(
//   core::pose::Pose & pose,
//   protocols::loops::Loop const & loop,
//   core::kinematics::MoveMap & mm
// );

#endif
