#include<iostream>
#include <sys/types.h>
#include <unistd.h>
#include <string>
#include <fstream> #include <time.h> #include <stdlib.h> 

//Rosetta Libraries
#include <protocols/init.hh> // Rosetta initialization
#include <core/fragment/FragmentIO.hh>
#include <core/fragment/FragSet.hh>
#include <core/conformation/Conformation.hh>
#include <core/options/option.hh>
#include <core/options/keys/in.OptionKeys.gen.hh>


// 
#include <disorder_remodel.hh>
#include <init_rosetta.hh>
#include <mr.hh>
#include <argument_parser.hh>

#ifdef USEMPI
#include <mpi_job_distribution.hh>
#endif
#include <batch_job.hh>

using namespace std;
using namespace core;

void usage() {
  cout<<"program -s score_file_path \
                 -i input_model_path \
                 -n # structure \
                 -r @flags (rosetta style) \
                 -p phaser_command_file (phaser command file) \
                 -sg (space group) \
                 -np (number of copies in ASU) \
                 [-c path_to_crystal_structure]"<<endl;
  cout<<"-s score_file_path [path to the AP-RDS file.]"<<endl;
  cout<<"-i model_file_path [path to coarse-grained model.]"<<endl;
  cout<<"-n # structure [default 10]"<<endl;
  cout<<"-r rosetta_flag file [path to the Rosetta flags(database, fragments)]"<<endl;
  cout<<"-p phaser command file [command as in standard Phaser]"<<endl;
  //cout<<"-ma maximum threshold [maximum Phaser (score default 7.5)]"<<endl;
  cout<<"-mi minimum threshold [minimum Phaser (score default 5.5)]"<<endl;
  cout<<"-u # successful models [# successful models after Phaser run (default 5.0)]"<<endl;
  cout<<endl<<endl;
  cout<<"Note:"<<endl;
  cout<<"please read the README file usage of phaser command"<<endl;
  exit(0);
}


int main (int argc, char* argv []) {
  map<string, string> args = ArgumentParser::argument_parser(argc, argv);

  Rosetta rosetta = ArgumentParser::rosetta_argument_parser(argc, argv);
  if(rosetta.get_init() == 0 || args.empty()) 
    usage();
  else
    cout<<"Rosetta database setup is completed."<<endl;

  float min_mr_threshold = 0.0, max_mr_threshold = 0.0;
  size_t num_success_models = 0;

  // MolecularReplacement mr = MolecularReplacement(args["-p"], args["-sg"], args["-np"]);
  MolecularReplacement mr = MolecularReplacement(args["-p"]);
  if(args["-p"].empty()) {
    cout<<"MR set is not executed."<<endl;
  }
  else {
    (args["-mi"].empty()) ? min_mr_threshold = 6.5: min_mr_threshold   = atof(args["-mi"].c_str());
    (args["-u"].empty())  ? num_success_models = 2: num_success_models = atoi(args["-u"].c_str());
  }
  
  DisorderModel disorder_model_(args["-s"], args["-i"]); 
  disorder_model_.set_num_of_models((args["-n"])); 
  disorder_model_.set_MR(mr);
  if(!args["-c"].empty()) {
    cout<<"Is this strange "<<args["-c"]<<endl;
    disorder_model_.set_native_pose(args["-c"]); 
  }

  if(disorder_model_.initialize(rosetta)) {
#ifdef USEMPI
    if(!args["-p"].empty()) {
      MPI_Job_Dist::mpi_start(argc, argv, disorder_model_, 
                              min_mr_threshold, num_success_models); 
    }
    else 
      MPI_Job_Dist::mpi_start(argc, argv, disorder_model_); 
#else
    if(!args["-p"].empty()) 
      batch_job::distribute_job(disorder_model_,min_mr_threshold, num_success_models);
    else
      batch_job::distribute_job(disorder_model_);
#endif
  }
  cout <<"Successfully finished." << endl;

  return EXIT_SUCCESS;
}

