// #include <utility/vector1.fwd.hh>
#include <anchor_relax.hh>
#include <protocols/loops/loops_main.hh>

#include <map>
#include <vector>

#include <simulated_annealing.hh>

using namespace std;
static numeric::random::RandomGenerator ruRG(20200);


SimulatedAnnealing::SimulatedAnnealing() {
}

void SimulatedAnnealing::initialization(int argc, char* argv[]) {
  protocols::abinitio::ClassicAbinitio::register_options();
  protocols::abinitio::AbrelaxApplication::register_options();
  protocols::init(argc, argv); //setup database is done 
  if (!SimulatedAnnealing::setup_fragments()) { 
    std::cout << "No fragment library!!!" <<std::endl;
    //exit 
  }
  //setup scoring function
  setup_score_functions();

  //initialize disorder regions
  string infile= "./test/S_00043496.dat";
  map <int, float> rsds_to_be_built;
  if (DisorderResidue::read_disorder_scores(infile, & rsds_to_be_built))
    disorder_residues_.generate_disorder_rsds(rsds_to_be_built);

  SimulatedAnnealing::Pose current_pose;
  string const input_pdb =  option[ OptionKeys::loops::input_pdb ]().name();
  core::io::pdb::centroid_pose_from_pdb(current_pose, input_pdb);
  SimulatedAnnealing::current_pose_ = current_pose;

  model_evaluation_.set_defaults(current_pose);
  RMover r_temp = RMover( SimulatedAnnealing::fragset_small_, 
                          SimulatedAnnealing::fragset_large_, 
                          SimulatedAnnealing::model_evaluation_.get_curr_scorefxn_op());
  SimulatedAnnealing::r_mover_ = r_temp; 
  cout << "Initialization is completed." << endl;
}



void SimulatedAnnealing::rebuild_wrong_regions( ) {
  Pose const start_pose = current_pose_;
  map<int, float> resi_rmsds = badregions_.get_residues_to_be_built();
  list<WrongPosition> fragments_to_be_replaced = badregions_.get_position_to_be_built();
  int m = 0;
  float initial_energy = (SimulatedAnnealing::model_evaluation_.get_curr_scorefxn())(current_pose_); 
  float final_energy = 0; 
  while(++m < 201) {
    // short_fragment_replacement(current_pose_, resi_rmsds);
    loop_like_fragment_replacement(current_pose_, fragments_to_be_replaced);
    current_pose_.dump_pdb(int2string(m) + ".pdb");
    // final_energy = (SimulatedAnnealing::model_evaluation_.get_curr_scorefxn())(current_pose_);
    // cout << "Model " << m << " " << initial_energy << " "
    //      << final_energy << " " << final_energy - initial_energy << endl;
    current_pose_ = start_pose;
  }
}


void SimulatedAnnealing::count_modified_residue_position( vector<int>& values, int pos) {
  for(int i = 0; i < int(values.size()); i++) {
    if(i == pos) {
      values[i]++;
      return;
    }
  }
}


string SimulatedAnnealing::int2string(int number) {
  stringstream ss;     //create a stringstream
  ss << number;        //add number to the stream
  return ss.str();     //return a string with the contents of the stream
}

void SimulatedAnnealing::print_counts(vector<int> counts) {
  for(int i = 0; i < int(counts.size()); i++) 
    cout<< i + 1 << " " << counts[i] << endl;
}

void SimulatedAnnealing::initialize_count(vector<int>& values, int size) {
  for(int i = 0; i < size; i++) 
    values.push_back(i);
} 

bool SimulatedAnnealing::setup_fragments( ) { 
  try {
    string frag9_file; 
    string frag3_file;  
    utility::vector1<int> frag_sizes( option[ OptionKeys::loops::frag_sizes ] );
    FileVectorOption frag_files(option[ OptionKeys::loops::frag_files ]);
    for(unsigned i =1; i<= frag_sizes.size(); i++) {
      if (frag_sizes[i] == 9) 
        frag9_file = frag_files[i];
      else if(frag_sizes[i] == 3)
        frag3_file = frag_files[2];
    } 
   
    if ((frag9_file == std::string("none")) && (frag3_file == std::string("none")))
      return false;

    core::fragment::FragmentIO fragIO;
    SimulatedAnnealing::fragset_small_ = fragIO.read_data( frag3_file ); 
    SimulatedAnnealing::fragset_large_ = fragIO.read_data( frag9_file ); 
    return true;
  }
  catch(exception& e) {
    cout<< e.what()<<endl;
    return false; 
  }
}


void SimulatedAnnealing::setup_score_functions() {
  fullatom_scorefxn = core::scoring::getScoreFunction(true);
  centroid_scorefxn = core::scoring::ScoreFunctionFactory::create_score_function("score3");
}

//@brief select the index to be modified ...
// int SimulatedAnnealing::routlette_wheel( map<int, float> residue_pairs) {
//   map<int, float>::iterator it;
//   float total;
//   for(it = residue_pairs.begin(); it != residue_pairs.end(); it++)
//     total += it->second;
//   float fortune = ruRG.uniform() * total;
//   for( it = residue_pairs.begin(); it != residue_pairs.end(); it++ ) {
//     fortune -= it->second;
//     if (fortune <= 0 ) {
//       return it->first;
//     }
//   }
//   return -1000; // never possible
// }

void choose_rsds_to_remodel() {
  list<DisorderResidue> dis_rsds = disorder_residues_.get_disorder_rsds(); 
  list<DisorderResidue>::iterator it_dis_rsds;
  float total_disorder_score;
  for( it_dis_rsds = dis_rsds.begin(); it_dis_rsds != dis_rsds.end(); it_dis_rsds++ ) {
    total_disorder_score += (*it_dis_rsds).get_disorder_score();
  }
  float fortune = ruRG.uniform() * total_disorder_score;
  for( it_dis_rsds = dis_rsds.begin(); it_dis_rsds != dis_rsds.end(); it_dis_rsds++ ) {
    fortune -= (*it_dis_rsds).get_disorder_score();
    if (fortune <= 0) 
      return (*it_dis_rsds)
  }
  return 0; //NULL
}

void SimulatedAnnealing::loop_like_fragment_replacement(SimulatedAnnealing::Pose &cur_pose, 
                                                list<WrongPosition> fragments_to_be_built) {
  list<WrongPosition>::iterator it;
  string ss = cur_pose.secstruct();
  cout << ss << std::endl; 
  int m = 1;
  for(it = fragments_to_be_built.begin(); it != fragments_to_be_built.end(); it++) {
    int start_rsd_pos, end_rsd_pos;
    start_rsd_pos = (*it).get_start_res_pos(); 
    end_rsd_pos   = (*it).get_end_res_pos();
    if(start_rsd_pos == end_rsd_pos) {
      start_rsd_pos--;
      end_rsd_pos++;
    }
    if (end_rsd_pos > start_rsd_pos) { //temporary
      float initial_energy = (*centroid_scorefxn)(cur_pose); 
      SimulatedAnnealing::r_mover_.apply(cur_pose, start_rsd_pos, end_rsd_pos);
      float final_energy = (*centroid_scorefxn)(cur_pose); 
      cout<<"ROS::starting position " << start_rsd_pos
          <<" " << end_rsd_pos <<" "<< initial_energy
          <<" " << final_energy<<endl;
    }
  }
  // AnchorRelax anchor_relax_(SimulatedAnnealing::fullatom_scorefxn, 1); 
  // anchor_relax_.apply(cur_pose);
}

void SimulatedAnnealing::short_fragment_replacement( 
                              SimulatedAnnealing::Pose &cur_pose,  
                              map<int, float> resi_rmsds) {
  unsigned num_iteration = 5;
  unsigned count = 0;
  string ss = cur_pose.secstruct();
  cout << ss << std::endl; 
  int success = 0;
  map<string, int> num_iter;
  while(count <= num_iteration ) {
    int pos = routlette_wheel(resi_rmsds);
    if (pos == -100000) continue;
    string sec_ss = check_ss(ss, pos);
    cout<< "ROS::selected residue " << pos << " " <<sec_ss <<endl;
    if (sec_ss.compare("H") == 0 || sec_ss.compare("E") == 0) 
      cout<< "Helix/Sheet" << endl;
    else {
      int start_rsd_pos, end_rsd_pos;
      SimulatedAnnealing::loop_model(cur_pose, pos, ss, start_rsd_pos, end_rsd_pos);
      cout << "ROS::Loop " << start_rsd_pos << " " << end_rsd_pos << endl;
      cout << "ROS::Loop " << ss.substr(start_rsd_pos - 1, end_rsd_pos - start_rsd_pos + 1) << endl;
      if (SimulatedAnnealing::max_loop_building(num_iter, start_rsd_pos, end_rsd_pos)) {
        float initial_energy = (SimulatedAnnealing::model_evaluation_.get_curr_scorefxn())(cur_pose); 
        SimulatedAnnealing::r_mover_.apply(cur_pose, start_rsd_pos, end_rsd_pos);
        float final_energy = (SimulatedAnnealing::model_evaluation_.get_curr_scorefxn())(cur_pose); 
        cout <<"ROS::Initial energy " << initial_energy << " final energy " << final_energy<< endl;
        success++;
      }
    }
    count++;
  }
  cout << "success rate " << success  << endl;
}

void SimulatedAnnealing::loop_model(Pose &cur_pose, int cur_rsd_pos, 
                          string ss, int & start_rsd_pos, int & end_rsd_pos) {
  
  start_rsd_pos = badregions_.get_start_pos(cur_rsd_pos);
  end_rsd_pos   = badregions_.get_end_pos(cur_rsd_pos);
  int count = cur_rsd_pos;
  while(count < end_rsd_pos) {
    if(ss.substr(count, 1).compare("L") == 0)
      count++;
    else
      break;
  }
  end_rsd_pos = count--;
  count = cur_rsd_pos;
  while(count > start_rsd_pos) {
    if(ss.substr(count, 1).compare("L") == 0)
      count--;
    else
      break;
  }
  start_rsd_pos = count++;
}

bool SimulatedAnnealing::max_loop_building(map<string, int> &num_iter, 
                  int start_rsd_pos, int end_rsd_pos) {

  map<string, int>::iterator it; 
  string const cur_res = int2string(start_rsd_pos) + "-" + int2string(end_rsd_pos);
  int count = 1;
  for(it=num_iter.begin(); it!=num_iter.end(); it++) {
    if ((*it).first == cur_res) 
      count = (*it).second;
  }
  if ( count > 3) // MAX = 3 now
    return false;
  else { 
    num_iter[cur_res] = count;
    return true;
  }
}

//@brief return secondary structure element
string SimulatedAnnealing::check_ss( string const sec_ss, int pos) {
  return sec_ss.substr(pos - 1, 1);
}

void SimulatedAnnealing::register_mr_option() {
  using namespace core::options;
  using namespace OptionKeys;
  option.add_relevant(mr::premr::mtz);
  option.add_relevant(mr::premr::F);
  option.add_relevant(mr::premr::SIGF);
  option.add_relevant(mr::premr::molecular_weight);
  option.add_relevant(mr::premr::estimated_rmsd);
  option.add_relevant(mr::premr::num_copies);
  option.add_relevant(mr::premr::command_file);
  option.add_relevant(mr::premr::low_threshold);
  option.add_relevant(mr::premr::high_threshold);
}
