#ifndef SIMULATED_ANNEALING 
#define SIMULATED_ANNEALING

#include <exception>
#include <string>
#include <map>
#include <vector>

#include <protocols/init.hh> // Rosetta initialization
#include <protocols/abinitio/AbrelaxApplication.hh>
#include <protocols/abinitio/ClassicAbinitio.hh>
#include <core/types.hh>

#include <core/options/option.hh>
#include <core/options/keys/loops.OptionKeys.gen.hh>

#include <core/pose/Pose.hh>
#include <core/io/pdb/pose_io.hh>
#include <core/fragment/FragmentIO.hh>
#include <core/fragment/FragSet.hh>
#include <protocols/abinitio/FragmentMover.hh>
#include <numeric/random/random.hh>


#include <core/scoring/ScoreFunction.fwd.hh>
#include <core/scoring/ScoreType.hh>
#include <core/scoring/ScoreFunctionFactory.hh>

#include <evaluation.hh>
#include <disorder_residue_position.hh>
#include <r_mover.hh>
#include <loop_wrapper.hh>


using namespace std;
using namespace core::options;
using namespace core::options::OptionKeys;



class SimulatedAnnealing {

  private: //typedef
    typedef core::pose::Pose Pose;
 
  private:  //variables
    core::fragment::FragSetOP fragset_small_;
    core::fragment::FragSetOP fragset_large_;
    core::pose::Pose current_pose_;
    RMover r_mover_;
    Model_Evaluation model_evaluation_;
    
    // RebuildFragment badregions_;
   
    DisorderResidues disorder_residues_;

    core::scoring::ScoreFunctionOP fullatom_scorefxn;
    core::scoring::ScoreFunctionOP centroid_scorefxn;


  public:
    
    SimulatedAnnealing();
    

    void initialization(int argc, char* argv[]);

    bool setup_fragments();

    void setup_score_functions();

    string check_ss( string const sec_ss, int pos); 

    void remodel_disorder_rsds();

    void rebuild_wrong_regions(); 
   
    string int2string(int number); 

    void print_counts(vector<int> counts);
    
    void initialize_count(vector<int>& values, int size); 

    void count_modified_residue_position( vector<int>& values, int pos); 
    
    void short_fragment_replacement(Pose &cur_pose,  map<int, float> resi_rmsds); 
    
    void loop_like_fragment_replacement(Pose &cur_pose, list<WrongPosition> fragments_to_be_built); 

    void loop_model(Pose &cur_pose, int cur_rsd_pos, string ss, int& start_rsd_pos, int& end_rsd_pos); 
    
    bool max_loop_building(map<string, int> &num_iter, int start_rsd_pos, int end_rsd_pos); 
    
    bool ss_frag_change(Pose &cur_pose, int pos, int frag_size); 
    
    int routlette_wheel( map<int, float> residue_pairs) ;
    
    static void register_mr_option();
    
};

# endif


