#include <structure_util.hh>

// float ModelUtility::CA_rmsd(const Pose &pose1, const Pose &pose2) {
//   using namespace core;
//   Size start ( 1 );
//   Size end ( std::min( pose1.total_residue(), pose2.total_residue() ) );
//   return CA_rmsd( pose1, pose2, start, end );
// }


float ModelUtility::CA_rmsd(const Pose & pose1, const Pose & pose2, vector<float> &rsd_rmsds) {
  const core::Size start1 = pose1.residue(1).seqpos(); 
  const core::Size end1   = pose1.total_residue(); 
  const core::Size start2 = pose2.residue(1).seqpos(); 
  const core::Size end2   = pose2.total_residue(); 
  if((end1 - start1) != (end2-start2)) {
    return -1;
  }
  else
    return CA_rmsd(pose1, pose2, start1, end1, rsd_rmsds);
}

float ModelUtility::CA_rmsd(const Pose & pose1, const Pose & pose2,
                            const core::Size start,
                            const core::Size end, 
                            vector<float> &rsd_rmsds) {
  using namespace core;
  // copy coords into Real arrays
  int natoms;
  FArray2D< core::Real > p1a;//( 3, pose1.total_residue() );
  FArray2D< core::Real > p2a;//( 3, pose2.total_residue() );
  fill_rmsd_coordinates( natoms, p1a, p2a, pose1, pose2, is_protein_CA );
  if ( (int) end > natoms ) 
    cout<< "WARNING: CA_rmsd out of range... range" << start << " " << end << " requested "
        << "but only " << natoms << " CA atoms found in fill_rmsd_coordinates " << std::endl;

  // if poses have different size or the range start..end doesn't fit we have
  // to get a different selection. would be nicer to that in
  // fill_rmsd_coordinates: best by refactoring the predicates to be a class
  // that can have some member variables that stores info's like ranges and
  // stuff quick hack instead of refactoring the whole RMSD code: refill Farray
  // with the appropriate range
  if ( pose1.total_residue() > end || pose2.total_residue() > end || start > 1 ) {
    FArray2D< core::Real > r1a( 3, end-start+1 );
    FArray2D< core::Real > r2a( 3, end-start+1 );
    for ( Size i = start; i<= (Size) std::min( (int)end, natoms); i++ ) {
      for ( Size d = 1; d<=3; d++ ) {
        r1a( d, i-start+1 ) = p1a( d, i );
        r2a( d, i-start+1 ) = p2a( d, i );
      }
    }
    natoms = end-start+1;
    //Real rms = numeric::model_quality::rms_wrapper( natoms, r1a, r2a );
    Real rms = rms_wrapper( natoms, r1a, r2a, rsd_rmsds );
    if(rms < 0.00001) rms = 0.0;
    return rms;
  }
  // Calc rms
  //Real rms = numeric::model_quality::rms_wrapper( natoms, p1a, p2a );
  Real rms = rms_wrapper( natoms, p1a, p2a, rsd_rmsds );
  if(rms < 0.00001) rms = 0.0;
  return rms;
} // CA_rmsd


float ModelUtility::rms_wrapper(int natoms,
                                FArray2D< numeric::Real > p1a,
                                FArray2D< numeric::Real > p2a, 
                                vector<float> &rsd_rmsds) {
  FArray1D< numeric::Real > ww( natoms, 1.0 );
  FArray2D< numeric::Real > uu( 3, 3, 0.0 );
  numeric::Real ctx;

  findUU( p1a, p2a, ww, natoms, uu, ctx );
  float fast_rms;
  calc_rms_fast(fast_rms, p1a, p2a, ww, natoms, ctx, rsd_rmsds);

  return fast_rms;
} // rms_wrapper

void ModelUtility::calc_rms_fast(float & rms_out,
                                  FArray2A< numeric::Real > xx,
                                  FArray2A< numeric::Real > yy,
                                  FArray1A< numeric::Real > ww,
                                  int npoints,
                                  numeric::Real ctx,
                                  vector<float> &rsd_rmsds) {
  xx.dimension( 3, npoints );
  yy.dimension( 3, npoints );
  ww.dimension( npoints );

  numeric::Real rms = 0.0;

  for ( int k = 1; k <= npoints; ++k ) {
    numeric::Real const ww_k = ww(k);
    for ( int j = 1; j <= 3; ++j ) {
      numeric::Real const xx_jk = xx(j,k);
      numeric::Real const yy_jk = yy(j,k);
      rms += ww_k * ( ( xx_jk * xx_jk ) + ( yy_jk * yy_jk ) );
    }
    //rsd_dist = 0.0;
  }
  rms -= 2 * ctx;
  // abs on next line catches small difference of large numbers that turn out
  // to be accidental small but negative
  rms_out = std::sqrt(std::abs(rms/npoints)); // return a float
} // calc_rms_fast


